#!/usr/bin/env python
# -*- coding: utf-8 -*-
########################################################################
#
# mamehist.py
# This file is part of Mamehist 0.2
# A simple reader utility for the MAME history.dat
#
# Copyright (c) 2009-2011 Elias Schwerdtfeger, http://www.tamagothi.de/
#
# Licensed under the terms of the pirate's license. You can do with
# this program whatever you like, as long as you do not sue me. If
# you want to sue me, ask me for a commercial license.
#
# Of course no warranty. This program may eat your hard drive, trash
# your data and traumatize your cat. It's your risk. 
#
########################################################################

import sys
import os
import os.path
import re
import locale

from codecs import EncodedFile
from formatter import AbstractFormatter, DumbWriter
from subprocess import Popen, PIPE

locale.setlocale(locale.LC_ALL, '')

# The directories to search for history.dat
DIRS = [ '/usr/share/games/xmame',
         '/usr/share/games/mame',
         '/usr/local/share/games/xmame',
         '/usr/local/share/games/mame' ]

# The name of the pager program
PAGER = os.getenv('PAGER', 'less')
LOCALENC = locale.getpreferredencoding()

# Scan for history.dat file
_DATNAME = 'history.dat'
_DATFILE = None
for aktdir in DIRS:
    aktname = os.path.join(aktdir, _DATNAME)
    if os.path.exists(aktname):
        _DATFILE = aktname
        break
DATFILE = os.getenv('HISTORYDAT', _DATFILE)
del aktdir
del aktname
if not DATFILE:
    exit('No %s file found, set HISTORYDAT in your envionment' % (_DATNAME,))
del _DATNAME
del _DATFILE

# Try to get number of columns from the envionment
COLUMNS = os.getenv('COLUMNS')
if COLUMNS == None:
    COLUMNS = os.getenv('COLS')
    if COLUMNS == None:
        COLUMNS = 80
COLUMNS -= 4

# Parse command line
if len(sys.argv) < 2:
    exit('Usage: %s romname' % (sys.argv[0],))
elif len(sys.argv) > 2:
    exit('Too many arguments')
else:
    term = sys.argv[1]

# Compile regexes for scanning the file
inf_re = re.compile(r'\$info\=([0-9a-z]*)', re.I)
bio_re = re.compile(r'\$bio', re.I)
end_re = re.compile(r'\$end', re.I)

# Initialize
state = 0
hfh = open(DATFILE, 'rU')
pro = None
of = None
af = None

# Main loop
for line in hfh:
    if state == 0:
        # State 0: ROM name not found, scanning
        m = inf_re.match(line)
        if m:
            found = m.group(1)
            games = found.split(',')
            if term in games:
                state = 1
    elif state == 1:
        # State 1: Scanning for start of bio section
        if bio_re.match(line):
            # Bio section found? Open pipe to the pager
            state = 2
            pro = Popen(PAGER, stdin=PIPE, shell=True)
            of = EncodedFile(pro.stdin, 'iso-8859-1', LOCALENC)
            af = AbstractFormatter(DumbWriter(of, COLUMNS))
    elif state == 2:
        # State 2: Printing
        if end_re.match(line):
            # End of bio section: Exit loop
            break
        line = line.strip()
        if len(line) == 0:
            af.end_paragraph(1)
        else:
            af.add_flowing_data(line)

# Cleanup
if pro:
    of.close()
    pro.wait()
else:
    sys.exit('No %s in the history.dat\n' % (term,))
    
